# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views to expose farmOS data directly
"""

from .master import FarmOSMasterView


def includeme(config):
    config.include("wuttafarm.web.views.farmos.users")
    config.include("wuttafarm.web.views.farmos.quantities")
    config.include("wuttafarm.web.views.farmos.asset_types")
    config.include("wuttafarm.web.views.farmos.units")
    config.include("wuttafarm.web.views.farmos.land_types")
    config.include("wuttafarm.web.views.farmos.land_assets")
    config.include("wuttafarm.web.views.farmos.structure_types")
    config.include("wuttafarm.web.views.farmos.structures")
    config.include("wuttafarm.web.views.farmos.animal_types")
    config.include("wuttafarm.web.views.farmos.animals")
    config.include("wuttafarm.web.views.farmos.groups")
    config.include("wuttafarm.web.views.farmos.plants")
    config.include("wuttafarm.web.views.farmos.log_types")
    config.include("wuttafarm.web.views.farmos.logs_activity")
    config.include("wuttafarm.web.views.farmos.logs_harvest")
    config.include("wuttafarm.web.views.farmos.logs_medical")
    config.include("wuttafarm.web.views.farmos.logs_observation")
